//
// Kodi Remote app v0.1 by Geert-Jan Kreileman
//
import QtQuick 1.1
import BasicUIControls 1.0
import qb.components 1.0
import "PlaylistItems.js" as PlaylistItemsJS

Screen {
	id: root

	screenTitleIconUrl: "drawables/kodiSystrayIcon.png"
	
	//Edit these settings:
	property string hostName: "localhost"
	property string portNum: "81"
	screenTitle: qsTr("Mediabox")
	//Stop editing here!
	
	property int forceUpdate: 0
	property int speed
	property int playTime
	property variant totalTime: []
	onHidden: {
		screenStateController.screenColorDimmedIsReachable = true;
		kodiPlayTimer.running = false;
		playTimer.running = false;
		kodiPlayInfoTimer.running = false;
		kodiPlaylistTimer.running = false;
	}
	onShown: {
		pageThrobber.visible = true;
		playTime = 0;
		totalTime = {"hours":0,"milliseconds":0,"minutes":0,"seconds":0};
		playTimer.running = true;
		kodiPlayTimer.running = true;
		kodiPlayInfoTimer.running = true;
		kodiPlaylistTimer.running = true;
		updatePlaylist();
		updateKodiPlayTimer();
		updateKodiPlayInfoTimer();
		screenStateController.screenColorDimmedIsReachable = false;
	}
	
	function simpleSynchronous(request) {
		pageThrobber.visible = true;
		var xmlhttp = new XMLHttpRequest();
		xmlhttp.open("GET", request, true);
		xmlhttp.timeout = 1500;
		xmlhttp.send();
		xmlhttp.onreadystatechange=function() {
			if (xmlhttp.readyState == 4) {
				if (xmlhttp.status == 200) {
					pageThrobber.visible = false;
					if (typeof(functie) !== 'undefined') {
						functie(parameter);
					}
				}
			}
		}
	}
	Throbber {
		id: pageThrobber
		visible: true
		z: 20
		anchors {
			horizontalCenter: parent.horizontalCenter
			verticalCenter: parent.verticalCenter
		}
	}
	StyledRectangle {
		id: nowPlaying
		width: nowPlayingImage.width+6
		height: nowPlayingImage.height+6
		radius: 3
		color: colors.background
		opacity: ((nowPlayingImage.height > 0)? 1.0:0.0)
		shadowColor: colors.shadow
		shadowPixelSize: 1
		anchors.top: parent.top
		anchors.left: parent.left
		anchors.leftMargin: 23
		anchors.topMargin: 30
		
		Image {
			id: nowPlayingImage
			source: ""
			width: 220
			height: ((sourceSize.height/sourceSize.width)*width) 
			anchors.top: parent.top
			anchors.left: parent.left
			anchors.leftMargin: 3
			anchors.topMargin: 3
			/*onClicked: {
				console.log("play/pauze");
				if (speed == 1) {
					//pause
					playButton.visible = true;
					pauseButton.visible = false;
					speed = 1;
				} else {
					//play
					playButton.visible = false;
					pauseButton.visible = true;
					speed = 1;
				}
				simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.PlayPause%22,%20%22params%22:%20{%20%22playerid%22:%201%20},%20%22id%22:%201}");
			}*/
			
			
		}
	}
	Text {
		id: itemText

		text: "-"
		font.pixelSize: 15
		font.family: qfont.regular.name
		font.bold: true
		color: colors.tileTextColor

		wrapMode: Text.WordWrap
		horizontalAlignment: Text.AlignHCenter
		anchors {
			top: nowPlaying.bottom
			topMargin: 15
			left: parent.left
			leftMargin: 35
		}
		width: 200
	}
	Text {
		id: itemTime

		text: "-"
		font.pixelSize: 13
		font.family: qfont.regular.name
		font.bold: false
		color: colors.tileTextColor

		wrapMode: Text.WordWrap
		horizontalAlignment: Text.AlignHCenter
		anchors {
			top: itemText.bottom
			topMargin: 3
			left: parent.left
			leftMargin: 35
		}
		width: 200
	}
	IconButton {
		id: prevButton
		anchors {
			left: parent.left
			leftMargin: 50
			top: itemTime.bottom
			topMargin: 10
		}

		iconSource: "drawables/left.png"
		onClicked: {
			simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.GoTo%22,%20%22params%22:%20{%22playerid%22:1,%20%22to%22:%20%22previous%22},%20%22id%22:%221%22}");
		}
	}
	IconButton {
		id: playButton
		anchors {
			left: prevButton.right
			leftMargin: 10
			top: itemTime.bottom
			topMargin: 10
		}
		visible: false

		iconSource: "drawables/play.png"
		onClicked: {
			console.log("play");
			playButton.visible = false;
			pauseButton.visible = true;
			speed = 0;
			simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.PlayPause%22,%20%22params%22:%20{%20%22playerid%22:%201%20},%20%22id%22:%201}");
		}
	}
	IconButton {
		id: pauseButton
		anchors {
			left: prevButton.right
			leftMargin: 10
			top: itemTime.bottom
			topMargin: 10
		}

		iconSource: "drawables/pause.png"
		onClicked: {
			console.log("pauze");
			playButton.visible = true;
			pauseButton.visible = false;
			speed = 1;
			simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.PlayPause%22,%20%22params%22:%20{%20%22playerid%22:%201%20},%20%22id%22:%201}");
		}
	}
	IconButton {
		id: stopButton
		anchors {
			left: playButton.right
			leftMargin: 10
			top: itemTime.bottom
			topMargin: 10
		}

		iconSource: "drawables/stop.png"
		onClicked: {
			simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.Stop%22,%20%22params%22:%20{%20%22playerid%22:%201%20},%20%22id%22:%201}");
		}
	}
	IconButton {
		id: nextButton
		anchors {
			left: stopButton.right
			leftMargin: 10
			top: itemTime.bottom
			topMargin: 10
		}

		iconSource: "drawables/right.png"
		onClicked: {
			console.log("next");
			simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.GoTo%22,%20%22params%22:%20{%22playerid%22:1,%20%22to%22:%20%22next%22},%20%22id%22:%221%22}");
		}
	}
	
	Component {
		id: brandListDelegate
		Item {
			width: 422
			height: 38
			/*Rectangle {
				id: testarea
				width: parent.width
				height: parent.height
				anchors.fill: parent
				anchors.topMargin: -10
				color: colors.ibListViewBckgNohighlight
			}*/
			MouseArea {
				id: mouse_area1
				z: 1
				property string kpiId: "kodiButton." + item
				anchors.fill: parent
				anchors.topMargin: -10
				onClicked: {
					//console.log("hoooi: "+item);
					playTime = 0;
					itemText.text = PlaylistItemsJS.items[item]['name'];
					simpleSynchronous("http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.GoTo%22,%20%22params%22:%20{%22playerid%22:1,%20%22to%22:%20"+item+"},%20%22id%22:%221%22}");
					forceUpdate = 1;
				}
			}
			Image {
				id: thumb
				source: PlaylistItemsJS.items[item]['thumb']
				fillMode: Image.PreserveAspectFit
				width: 30
				height: 30
				anchors {
					top: parent.top
					topMargin: -3
				}
				
			}
			Text {
				id: listItemText

				text: PlaylistItemsJS.items[item]['name']
				font.pixelSize: 15
				font.family: qfont.regular.name
				font.bold: ((itemText.text == PlaylistItemsJS.items[item]['name'])? true:false)
				color: ((itemText.text == PlaylistItemsJS.items[item]['name'])? colors.wifiActiveNetwork:colors.foreground)

				wrapMode: Text.WrapAnywhere
				maximumLineCount: 1
				elide: Text.ElideRight
				verticalAlignment: Text.AlignVCenter
				anchors {
					left: thumb.right
					leftMargin: 10
				}
				width: boilerScrollableSimpleList.width - 166
			}
			Text {
				id: itemTime
				text: PlaylistItemsJS.items[item]['time']
				font.pixelSize: 15
				font.family: qfont.regular.name
				font.bold: ((itemText.text == PlaylistItemsJS.items[item]['name'])? true:false)
				color: ((itemText.text == PlaylistItemsJS.items[item]['name'])? colors.wifiActiveNetwork:colors.foreground)

				wrapMode: Text.WrapAnywhere
				horizontalAlignment: Text.AlignHRight
				verticalAlignment: Text.AlignVCenter
				maximumLineCount: 1
				width: 40
				elide: Text.ElideRight
				anchors {
					right: parent.right
					rightMargin: 0
				}
			}
		}
	}
	
	ScrollableSimpleList {
		id: boilerScrollableSimpleList
		width: 500
		height: 355
		itemsPerPage: 8
		delegate: brandListDelegate
		anchors.top: parent.top
		anchors.right: parent.right
		anchors.rightMargin: 30
		anchors.topMargin: 30

		Throbber {
			id: throbber
			visible: false
			anchors {
				horizontalCenter: parent.horizontalCenter
				horizontalCenterOffset: -26
				verticalCenter: parent.verticalCenter
			}
		}

		Text {
			id: noConnectionText
			visible: false
			anchors {
				horizontalCenter: parent.horizontalCenter
				horizontalCenterOffset: -26
				verticalCenter: parent.verticalCenter
			}
			color: colors.taNoInternet
			font.family: qfont.italic.name
			font.pixelSize: 15
			text: qsTr("no-connection")
		}
	}
	function pad(n, width) {
		n = n + '';
		return n.length >= width ? n : new Array(width - n.length + 1).join('0') + n;
	}
	function updatePlayTimer() {
		if (speed == 1) {
			playTime = playTime +1;
			itemTime.text = Math.floor(playTime/60)+":"+pad((playTime-(Math.floor(playTime/60)*60)), 2)+" / "+((totalTime['hours'] != "0")? totalTime['hours']+":":"")+totalTime['minutes']+":"+pad(totalTime['seconds'], 2);
		}
		if (playTime < 0) {
			itemTime.text = "-";
		}
	}
	function updateKodiPlayTimer() {
		var xmlhttp = new XMLHttpRequest();
		xmlhttp.onreadystatechange=function() {
			if (xmlhttp.readyState == 4) {
				if (xmlhttp.status == 200) {
					var response = JSON.parse(xmlhttp.responseText);
					playTime = ((response['result']['time']['hours']*3600)+(response['result']['time']['minutes']*60)+response['result']['time']['seconds'])-1;
					totalTime = response['result']['totaltime'];
					speed = response['result']['speed'];
					if ((response['result']['totaltime']['hours']+response['result']['totaltime']['minutes']+response['result']['totaltime']['seconds']) == 0) {
						//speelt niks
						speed = 0;
					}
					if (speed == 1) {
						playButton.visible = false;
						pauseButton.visible = true;
					} else {
						playButton.visible = true;
						pauseButton.visible = false;
					}
					pageThrobber.visible = false;
				}
			}
		}
		xmlhttp.open("GET", "http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%222.0%22,%22method%22:%22Player.GetProperties%22,%22params%22:{%22playerid%22:1,%22properties%22:[%22time%22,%22totaltime%22,%22type%22,%22speed%22]},%22id%22:%221%22}", true);
		xmlhttp.send();
	}
	function updateKodiPlayInfoTimer() {
		var xmlhttp = new XMLHttpRequest();
		xmlhttp.onreadystatechange=function() {
			if (xmlhttp.readyState == 4) {
				if (xmlhttp.status == 200) {
					var response = JSON.parse(xmlhttp.responseText);
					if (itemText.text != response['result']['item']['title'] || forceUpdate == 1) {
						//Only update when needed
						itemText.text = response['result']['item']['title'];
						nowPlayingImage.source = decodeURIComponent(response['result']['item']['thumbnail'].replace("image://", "").slice(0, -1)).replace("https://", "http://");
						updateKodiPlayTimer();
						//if there is a new title, the playlist will be different too
						updatePlaylist();
						//console.log("kodiplayinfotimer, force: "+forceUpdate+" response: "+xmlhttp.responseText);
						forceUpdate = 0;
					}
				}
			}
		}
		xmlhttp.open("GET", "http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Player.GetItem%22,%20%22params%22:%20{%20%22properties%22:%20[%22title%22,%20%22album%22,%20%22artist%22,%20%22season%22,%20%22episode%22,%20%22duration%22,%20%22showtitle%22,%20%22tvshowid%22,%20%22thumbnail%22,%20%22file%22,%20%22fanart%22,%20%22streamdetails%22],%20%22playerid%22:%201%20},%20%22id%22:%20%22VideoGetItem%22}", true);
		xmlhttp.send();
		
		itemTime.text = Math.floor(playTime/60)+":"+pad((playTime-(Math.floor(playTime/60)*60)), 2)+" / "+((totalTime['hours'] != "0")? totalTime['hours']+":":"")+totalTime['minutes']+":"+pad(totalTime['seconds'], 2);
	}
	function objToString (obj) {
			var str = '';
			for (var p in obj) {
				if (obj.hasOwnProperty(p)) {
					str += p + '::' + obj[p] + '\n';
				}
			}
			return str;
		}
	function updatePlaylist() {
		var xmlhttp = new XMLHttpRequest();
		xmlhttp.onreadystatechange=function() {
			if (xmlhttp.readyState == 4) {
				if (xmlhttp.status == 200) {
					var response = JSON.parse(xmlhttp.responseText);
					boilerScrollableSimpleList.removeAll();
					if (response['result']['limits']['total'] > 0) {
						for (var i = 0; i < response['result']['items'].length; i++) {
							PlaylistItemsJS.items[i] = {"time": Math.floor(response['result']['items'][i]['runtime']/60)+":"+pad((response['result']['items'][i]['runtime']-(Math.floor(response['result']['items'][i]['runtime']/60)*60)), 2), "name": response['result']['items'][i]['label'], "thumb": decodeURIComponent(response['result']['items'][i]['thumbnail'].replace("image://", "").slice(0, -1)).replace("https://", "http://")};
							boilerScrollableSimpleList.addDevice(i);
						}
						boilerScrollableSimpleList.refreshView();
						if (boilerScrollableSimpleList.currentPage == -1) {
							boilerScrollableSimpleList.scrollToPage(0);
						}
						//console.log("dit is de playlist: ");
						//console.log(objToString(playlistItemsJS.items));
					} else {
						boilerScrollableSimpleList.addDevice(itemText.text);
					}
				}
			}
		}
		xmlhttp.open("GET", "http://"+hostName+":"+portNum+"/jsonrpc?request={%22jsonrpc%22:%20%222.0%22,%20%22method%22:%20%22Playlist.GetItems%22,%20%22params%22:%20{%20%22properties%22:%20[%20%22runtime%22,%20%22showtitle%22,%20%22season%22,%20%22thumbnail%22,%20%22title%22,%20%22artist%22%20],%20%22playlistid%22:%201},%20%22id%22:%201}", true);
		xmlhttp.send();
	}
	
	Timer {
		id: kodiPlaylistTimer
		interval: 30000
		triggeredOnStart: true
		running: false
		repeat: true
		onTriggered: updatePlaylist()
	}
	Timer {
		id: kodiPlayInfoTimer
		interval: 5500
		triggeredOnStart: true
		running: false
		repeat: true
		onTriggered: updateKodiPlayInfoTimer()
	}
	Timer {
		id: kodiPlayTimer
		interval: 5000
		triggeredOnStart: true
		running: false
		repeat: true
		onTriggered: updateKodiPlayTimer()
	}
	Timer {
		id: playTimer
		interval: 1000
		triggeredOnStart: true
		running: false
		repeat: true
		onTriggered: updatePlayTimer()
	}
}
